//+------------------------------------------------------------------+
//|                                                     TMA Band.mq4 |
//|                                                       ~bull.bear |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "~bull.bear~"
#property link      " "

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Yellow
#property indicator_color2 Yellow
#property indicator_color3 Yellow
#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 1
#property indicator_style1 2
#property indicator_style2 2
#property indicator_style3 2

//Input parameters
extern int       Periods=24;
extern int       ApplyTo=0;
extern double    AtrMultiplier = 2.0;
extern int       AtrPeriod = 100;

//Apply To like "Applied price enumeration"
//0 Close price. 
//1 Open price. 
//2 High price. 
//3 Low price. 
//4 Median price, (high+low)/2. 
//5 Typical price, (high+low+close)/3. 
//6 Weighted close price, (high+low+close+close)/4

//Indicator buffers
double tma[];
double UBand[];
double LBand[];

//Global variables
int    weights[];
double divisor;


//--------------------------------------------------
int init()
{
  SetIndexStyle(0,DRAW_LINE);
  SetIndexBuffer(0,tma);

  SetIndexStyle(1,DRAW_LINE);
  SetIndexBuffer(1,UBand);

  SetIndexStyle(2,DRAW_LINE);
  SetIndexBuffer(2,LBand);

  IndicatorShortName( "TMA(" + Periods + ")" );

  //Calculate weigths
  ArrayResize(weights,Periods);

  double dPeriods = Periods;
  int to=MathCeil(dPeriods/2.0);

  for(int i=0; i<to; i++)
  {
    weights[i]            = i+1;
    weights[Periods-1-i]  = i+1;
  }

  divisor = 0.0;
  for(int j=0; j<Periods; j++)
    divisor += weights[j];

}

//--------------------------------------------------
int deinit()
{
}

//-------------------------------------------------------
int start()
{
  int    counted_bars = IndicatorCounted();
  if( counted_bars < 0 )
    return(-1);
  
  //Recalculate last bar
  if( counted_bars > 0 )
    counted_bars++;
  
  int limit = Bars - counted_bars;
  for(int i=limit-1; i>=0; i--)
  {
    double tma_val = 0.0;
    for(int j=0; j<Periods; j++)
    {
      double price = getPrice(ApplyTo, i+Periods-j);
      tma_val += price*weights[j];
    }
    
    double mid = tma_val/divisor;
    double dblRange = iATR(NULL,0,AtrPeriod,i);
    tma[i] = mid;
    UBand[i] = mid + (AtrMultiplier*dblRange);
    LBand[i] = mid - (AtrMultiplier*dblRange);   
  }
}

//--------------------------------------------------------
double getPrice(int priceType, int index)
{
  double price = 0.0;
  
  switch(priceType)
  {
    case PRICE_OPEN    : price = Open[index];
                         break;
  
    case PRICE_HIGH    : price = High[index];
                         break;
  
    case PRICE_LOW     : price = Low[index];
                         break;
  
    case PRICE_MEDIAN  : price = (High[index]+Low[index])/2.0;
                         break;
  
    case PRICE_TYPICAL : price = (High[index]+Low[index]+Close[index])/3.0;
                         break;

    case PRICE_WEIGHTED: price = (High[index]+Low[index]+2*Close[index])/4.0;
                         break;

    
    case PRICE_CLOSE   : 
    default            : price = Close[index];
                         break;

  }
  return(price);
}
//+------------------------------------------------------------------+